import { PublicKey } from '@solana/web3.js';
import { AuctionHouse, LazyPurchase, Purchase } from '../models';
import { Operation, OperationHandler } from '../../../types';
declare const Key: "FindPurchasesOperation";
/**
 * Finds Purchases by multiple criteria.
 * You can use any combination of keys.
 * Supports only 3 criteria at the same time including
 * the required `auctionHouse` attribute.
 *
 * ```ts
 * // Find all purchases in an Auction House.
 * const purchases = await metaplex
 *   .auctionHouse()
 *   .findPurchases({ auctionHouse });
 *
 * // Find purchases by buyer and mint.
 * const purchases = await metaplex
 *   .auctionHouse()
 *   .findPurchases({ auctionHouse, buyer, mint });
 *
 * // Find purchases by metadata.
 * const purchases = await metaplex
 *   .auctionHouse()
 *   .findPurchases({ auctionHouse, metadata });
 *
 * // Find purchases by seller and buyer.
 * const purchases = await metaplex
 *   .auctionHouse()
 *   .findPurchases({ auctionHouse, seller, buyer });
 * ```
 *
 * @group Operations
 * @category Constructors
 */
export declare const findPurchasesOperation: import("../../../types").OperationConstructor<FindPurchasesOperation, "FindPurchasesOperation", FindPurchasesInput, FindPurchasesOutput>;
/**
 * @group Operations
 * @category Types
 */
export type FindPurchasesOperation = Operation<typeof Key, FindPurchasesInput, FindPurchasesOutput>;
/**
 * @group Operations
 * @category Inputs
 */
export type FindPurchasesInput = {
    /** A model of the Auction House related to these listings. */
    auctionHouse: AuctionHouse;
    /** The address of a buyer to search by. */
    buyer?: PublicKey;
    /** The address of a seller to search by. */
    seller?: PublicKey;
    /**
     * The address of metadata to search by.
     * Ignored when mint provided.
     */
    metadata?: PublicKey;
    /** The address of a mint to search by. */
    mint?: PublicKey;
};
/**
 * @group Operations
 * @category Outputs
 */
export type FindPurchasesOutput = (Purchase | LazyPurchase)[];
/**
 * @group Operations
 * @category Handlers
 */
export declare const findPurchasesOperationHandler: OperationHandler<FindPurchasesOperation>;
export {};
